#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (C) 2019 tribe29 GmbH - License: GNU General Public License v2
# This file is part of Checkmk (https://checkmk.com). It is subject to the terms and
# conditions defined in the file COPYING, which is part of this source code package.

# snmpEngineTime : The number of seconds since the value of the snmpEngineBoots object last changed. When
#                  incrementing this object's value would cause it to exceed its maximum, snmpEngineBoots is
#                  incremented as if a re-initialization had occurred, and this object's value consequently
#                  reverts to zero.
# snmpEngineBoots: The number of times that the SNMP engine has (re-)initialized itself since snmpEngineID was
#                  last configured. On Cisco Device this gives the number of reboots
#
# 2022-03-06: extended to use snmpEngineTime if available
# 2022-03-07: fixed if snmpEngineTime == 0
# 2022-03-08: renamed to snmp_uptime_extended to avoid caching issues with the original check
# 2023-08-05: made parse function more stable (try/except/else)
# 2024-06-16: modified imports for CMK 2.3

from typing import Optional

from cmk.base.plugins.agent_based.agent_based_api.v1 import exists, register, SNMPTree
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import StringTable
from cmk.plugins.lib import uptime


def parse_snmp_uptime_extended(string_table: StringTable) -> Optional[uptime.Section]:
    """
    >>> parse_snmp_uptime_extended([['2297331594', '', '']])
    Section(uptime_sec=22973315, message=None)
    >>> parse_snmp_uptime_extended([['124:21:26:42.03', '124:21:29:01.14', '']])
    Section(uptime_sec=10790941, message=None)
    >>> None is parse_snmp_uptime_extended([[u'', u'Fortigate 80C', 'Whatever']])  # nonsense
    True
    >>> parse_snmp_uptime_extended([['2297331594', '', '313671']])
    Section(uptime_sec=313671, message=None)
    """
    try:
        sysUpTime, hrSystemUptime, snmpEngineTime = string_table[0]
    except (ValueError, IndexError):
        return

    ticks = sysUpTime or hrSystemUptime

    if len(ticks) < 3:
        return

    try:
        value = int(snmpEngineTime)
    except ValueError:
        pass
    else:
        if value > 0:
            return uptime.Section(int(snmpEngineTime), None)

    try:
        return uptime.Section(int(ticks[:-2]), None)
    except ValueError:
        pass

    try:
        days, h, m, s = ticks.split(":")
    except ValueError:
        pass
    else:
        return uptime.Section(
            (int(days) * 86400) + (int(h) * 3600) + (int(m) * 60) + int(float(s)),
            None,
        )


register.snmp_section(
    name="snmp_uptime_extended",
    parsed_section_name="uptime",
    parse_function=parse_snmp_uptime_extended,
    supersedes=['snmp_uptime'],
    fetch=SNMPTree(
        base=".1.3.6.1",
        oids=[
            # On Linux appliances: .1.3.6.1.2.1.1.3.0    means uptime of snmpd
            #                      .1.3.6.1.2.1.25.1.1.0 means system uptime
            "2.1.1.3",  # DISMAN-EVENT-MIB::sysUpTime
            "2.1.25.1.1",  # HOST-RESOURCES-MIB::hrSystemUptime
            "6.3.10.2.1.3",  # SNMP-FRAMEWORK-MIB::snmpEngineTime
        ],
    ),
    detect=exists(".1.3.6.1.2.1.1.1.0"),
)

